
from gettext import ngettext, gettext as _

import gobject
import gtk
import gnomevfs
import gnomevfs.async
try:
    import gconf
except ImportError:
    pass

from gimmie_base import Item


class TrashMonitor(gobject.GObject):
    '''
    Object which monitors the trash can.  This was mostly ported from
    trashapplet.
    '''

    __gsignals__ = {
        "count-changed" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_INT,))
        }

    def __init__(self):
        gobject.GObject.__init__(self)
        self.volume_monitor = gnomevfs.VolumeMonitor()
        self.trash_dirs = {}     # volume_uri : trash_uri
        self.trash_counts = {}   # trash_uri : trash count
        self.trash_monitors = {} # trash_uri : monitor handle
        self.item_count = 0
        self.notify_id = None

        self.volume_monitor.connect("volume_mounted", lambda m, v: self.add_volume(v))
        self.volume_monitor.connect("volume_pre_unmount", lambda m, v: self.remove_volume(v))

        for volume in self.volume_monitor.get_mounted_volumes():
            self.add_volume(volume)

    def add_volume(self, volume):
        uri = volume.get_activation_uri()
        if volume.handles_trash() and uri not in self.trash_dirs:
            gnomevfs.async.find_directory(near_uri_list = [gnomevfs.URI(uri)],
                                          kind = gnomevfs.DIRECTORY_KIND_TRASH,
                                          create_if_needed = False,
                                          find_if_needed = True,
                                          permissions = 0777,
                                          callback = self._find_directory,
                                          user_data = uri)

    def remove_volume(self, volume):
        volume_uri = volume.get_activation_uri()
        if volume_uri not in self.trash_dirs:
            return

        trash_uri = self.trash_dirs[volume_uri]
        monitor_handle = self.trash_monitors[trash_uri]
        count = self.trash_counts[volume_uri]

        gnomevfs.monitor_cancel(monitor_handle)

        del self.trash_monitors[trash_uri]
        del self.trash_dirs[volume_uri]
        del self.trash_counts[volume_uri]

        if count > 0:
            self.trash_changed_queue_notify()

    def _find_directory(self, handle, results, volume_uri):
        # FIXME: Support multiple trash directories per volume?
        for result in results:
            if result[1] == gnomevfs.NotSupportedError:
                continue

            trash_uri = str(result[0])
            if trash_uri in self.trash_monitors.keys():
                continue

            self.trash_dirs[volume_uri] = trash_uri
            self.trash_monitors[trash_uri] = gnomevfs.monitor_add(trash_uri,
                                                                  gnomevfs.MONITOR_DIRECTORY,
                                                                  self._trash_dir_changed)
            # simulate a change to update item count
            self._trash_dir_changed(trash_uri, None, gnomevfs.MONITOR_EVENT_CHANGED)
            break

    def _trash_dir_changed(self, monitor_uri, info_uri, event_type):
        count = 0
        try:
            for file in gnomevfs.open_directory(monitor_uri):
                if file.name not in (".", ".."):
                    count += 1
        except gnomevfs.NotFoundError:
            pass
        self.trash_counts[monitor_uri] = count
        self._trash_changed_queue_notify()
        return 1

    def _trash_changed_queue_notify(self):
        if not self.notify_id:
            self.notify_id = gobject.idle_add(self._trash_changed_idle)

    def _trash_changed_idle(self):
        item_count = 0
        for (volume, count) in self.trash_counts.items():
            item_count += count

        if self.item_count != item_count:
            self.item_count = item_count
            self.emit("count_changed", item_count)

        self.notify_id = None
        return False

    def empty(self):
        source_uris = [gnomevfs.URI(uri) for uri in self.trash_dirs.values()]
        gnomevfs.async.xfer(source_uri_list = source_uris,
                            target_uri_list = [],
                            xfer_options = gnomevfs.XFER_EMPTY_DIRECTORIES,
                            error_mode = gnomevfs.XFER_ERROR_MODE_ABORT,
                            overwrite_mode = gnomevfs.XFER_OVERWRITE_MODE_REPLACE,
                            progress_update_callback = self._empty_progress,
                            update_callback_data = None)

    def _empty_progress(self, handle, info, data=None):
        # FIXME: show progress/cancellation?
        return 1

    def is_empty(self):
        return self.item_count == 0

    def get_count(self):
        return self.item_count

    def recheck_trash_dirs(self):
        for volume in self.volume_monitor.get_mounted_volumes():
            self.add_volume(volume)


class TrashItem(Item):
    '''
    An item which monitors the trash can state
    '''
    def __init__(self):
        Item.__init__(self, name=_("Trash"), uri="trash:///", icon="gnome-fs-trash-full")
        self.empty = trash_monitor.is_empty()
        trash_monitor.connect("count-changed", self._count_changed)

    def get_icon(self, icon_size):
        # FIXME: accept icon?
        if trash_monitor.is_empty():
            self.icon = "gnome-fs-trash-empty"
        else:
            self.icon = "gnome-fs-trash-full"
        return Item.get_icon(self, icon_size)

    def _count_changed(self, monitor, count, data=None):
        if (count == 0) != self.empty:
            self.empty = (count == 0)
            self.emit("reload")

    def populate_popup(self, menu):
        open = gtk.ImageMenuItem(gtk.STOCK_OPEN)
        open.connect("activate", lambda w: self.open())
        open.show()
        menu.append(open)

        empty = gtk.MenuItem(_("_Empty Trash"), use_underline=True)
        empty.set_sensitive(not self.empty)
        empty.connect("activate", lambda w: self.do_empty())
        empty.show()
        menu.append(empty)

    def get_comment(self):
        return ngettext("%d item", "%d items", trash_monitor.get_count()) \
               % trash_monitor.get_count()

    def get_tooltip(self):
        ### FIXME: Tooltips are cached currently, so displaying the count always shows 0
        #return _("%s (%s)") % (self.get_name(), self.get_comment())
        return self.get_name()

    def get_is_opened(self):
        return True

    def do_empty(self):
	try:
            client = gconf.client_get_default()
            if not client.get_bool("/apps/nautilus/preferences/confirm_trash"):
                return True
	except NameError:
            pass

        dialog = gtk.MessageDialog(type = gtk.MESSAGE_WARNING,
                                   flags = gtk.DIALOG_MODAL,
                                   message_format = _("Empty all of the items from the trash?"))
        dialog.format_secondary_text(_("If you choose to empty the trash, all items in it will be permanently lost.  Please note that you can also delete them separately."))
        dialog.set_wmclass("empty_trash", "Nautilus");
        # FIXME: Set transient
        dialog.realize()

        dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)

        button = gtk.Button(label=_("_Empty Trash"), use_underline=True)
        button.set_property("can-default", True)
        button.show()
        dialog.add_action_widget(button, gtk.RESPONSE_ACCEPT)
        dialog.set_default_response(gtk.RESPONSE_ACCEPT)

        if dialog.run() == gtk.RESPONSE_ACCEPT:
            trash_monitor.empty()

        dialog.destroy()

    def _async_progress(self, handle, info, xfer_opts):
        if info.status == gnomevfs.XFER_PROGRESS_STATUS_VFSERROR:
            uri = gnomevfs.URI(info.source_name)

            if xfer_opts & gnomevfs.XFER_REMOVESOURCE:
                msg = _("Error while moving.")
                msg2 = _('Cannot move "%s" to the trash because you do not have permissions to change it or its parent folder.' % uri.short_name)
            elif xfer_opts & gnomevfs.XFER_DELETE_ITEMS:
                msg = _("Error while deleting.")
                msg2 = _('"%s" cannot be deleted because you do not have permissions to modify its parent folder.') % uri.short_name
            else:
                msg = _("Error while performing file operation.")
                msg2 = _('Cannot perform file operation %d on "%s".')  % (xfer_opts, uri.short_name)

            dialog = gtk.MessageDialog(type = gtk.MESSAGE_ERROR,
                                       message_format = msg)
            dialog.format_secondary_text(msg2)

            if info.files_total > 1:
                button = gtk.Button(label=_("_Skip"))
                button.show()
                dialog.add_action_widget(button, gtk.RESPONSE_REJECT)

            dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)

            button = gtk.Button(label=_("_Retry"))
            button.set_property("can-default", True)
            button.show()
            dialog.add_action_widget(button, gtk.RESPONSE_ACCEPT)
            dialog.set_default_response(gtk.RESPONSE_ACCEPT)

            response = dialog.run()
            dialog.destroy()

            if response == gtk.RESPONSE_ACCEPT:
                return gnomevfs.XFER_ERROR_ACTION_RETRY
            elif response == gtk.RESPONSE_REJECT:
                return gnomevfs.XFER_ERROR_ACTION_SKIP

            return gnomevfs.XFER_ERROR_ACTION_ABORT

        return 1

    def _async_progress_sync(self, info, data=None):
        # Dummy function. All code to handle error conditions is in
        # self._async_progress()
        return 1

    def _find_directory(self, handle, results, uri_list):
        # Recheck for any Trash dirs we may have added
        trash_monitor.recheck_trash_dirs()

        source_uri_list = []
        target_uri_list = []
        unmovable_uri_list = []
        for i in xrange(len(results)):
            trash_uri, error = results[i]
            source_uri = uri_list[i]

            # error is None if Trash directory is successfully found
            if error == None:
                source_uri_list.append(source_uri)
                target_uri_list.append(trash_uri.append_file_name(source_uri.short_name))
            else:
                unmovable_uri_list.append(source_uri)

        if len(source_uri_list) > 0:
            gnomevfs.async.xfer(source_uri_list, target_uri_list,
                                xfer_options = gnomevfs.XFER_REMOVESOURCE |
                                               gnomevfs.XFER_RECURSIVE,
                                error_mode = gnomevfs.XFER_ERROR_MODE_QUERY,
                                overwrite_mode = gnomevfs.XFER_OVERWRITE_MODE_REPLACE,
                                progress_update_callback = self._async_progress,
                                update_callback_data = gnomevfs.XFER_REMOVESOURCE,
                                progress_sync_callback = self._async_progress_sync)

        num_files = len(unmovable_uri_list)
        if (num_files > 0):
            if len(source_uri_list) == 0:
                msg = _("Cannot move items to trash, do you want to delete them immediately?")
                msg2 = _("None of the %d selected items can be moved to the Trash") % num_files
            else:
                msg = _("Cannot move some items to trash, do you want to delete these immediately?")
                msg2 = _("%d of the selected items cannot be moved to the Trash") % num_files

            dialog = gtk.MessageDialog(type = gtk.MESSAGE_QUESTION,
                                       flags = gtk.DIALOG_MODAL,
                                       message_format = msg)

            dialog.format_secondary_text(msg2)

            dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)

            button = gtk.Button(label=_("_Delete"), use_underline=True)
            button.set_property("can-default", True)
            button.show()
            dialog.add_action_widget(button, gtk.RESPONSE_ACCEPT)
            dialog.set_default_response(gtk.RESPONSE_ACCEPT)

            if dialog.run() == gtk.RESPONSE_ACCEPT:
                gnomevfs.async.xfer(source_uri_list = unmovable_uri_list,
                                    target_uri_list = [],
                                    xfer_options = gnomevfs.XFER_DELETE_ITEMS,
                                    error_mode = gnomevfs.XFER_ERROR_MODE_QUERY,
                                    overwrite_mode = gnomevfs.XFER_OVERWRITE_MODE_REPLACE,
                                    progress_update_callback = self._async_progress,
                                    update_callback_data = gnomevfs.XFER_DELETE_ITEMS,
                                    progress_sync_callback = self._async_progress_sync)
            dialog.destroy()

    def handle_drag_data_received(self, selection, target_type):
        uri_list = []
        for uri in selection.get_uris():
            uri_list.append(gnomevfs.URI(uri))

        gnomevfs.async.find_directory(near_uri_list = uri_list,
                                      kind = gnomevfs.DIRECTORY_KIND_TRASH,
                                      create_if_needed = True,
                                      find_if_needed = False,
                                      permissions = 0777,
                                      callback = self._find_directory,
                                      user_data = uri_list)

#
# Globals
#

trash_monitor = TrashMonitor()
